@@ -5,19 +5,22 @@
 class Host:
     """Represents a virtual machine or container in PVE."""
 
-    def __init__(self, vmid, hostname, ip_address, pve_type):
-        self.hostname = hostname
-        self.ip_address = ip_address
-        self.vmid = vmid
-        self.pve_type = pve_type
+    def __init__(self, vmid, hostname, ipv4_address, ipv6_address, pve_type):
+        self.hostname = str(hostname)
+        self.ipv4_address = str(ipv4_address)
+        self.ipv6_address = str(ipv6_address)
+        self.vmid = str(vmid)
+        self.pve_type = str(pve_type)
         self.labels = {}
-        self.labels["__meta_pve_ip"] = ip_address
-        self.labels["__meta_pve_name"] = hostname
-        self.labels["__meta_pve_type"] = pve_type
-        self.labels["__meta_pve_vmid"] = str(vmid)
+        self.add_label("ipv4", ipv4_address)
+        self.add_label("ipv6", ipv6_address)
+        self.add_label("name", hostname)
+        self.add_label("type", pve_type)
+        self.add_label("vmid", vmid)
 
     def __str__(self):
-        return f"{self.hostname}({self.vmid}): {self.pve_type} {self.ip_address}"
+        return f"{self.hostname}({self.vmid}): {self.pve_type} \
+                  {self.ipv4_address} {self.ipv6_address}"
 
     def add_label(self, key, value):
         key = key.replace("-", "_").replace(" ", "_")
@@ -33,6 +36,20 @@ class HostList:
     def __init__(self):
         self.hosts = []
 
+    def __eq__(self, other):
+        if not isinstance(other, HostList):
+            return False
+
+        if len(other.hosts) != len(self.hosts):
+            return False
+
+        for host in self.hosts:
+            if other.host_exists(host):
+                continue
+            return False
+
+        return True
+
     def clear(self):
         self.hosts = []
 
